from dataclasses import dataclass
from typing import Any, Type

from docx import Document
from docx.enum.text import WD_PARAGRAPH_ALIGNMENT
from docx.shared import Pt, Inches

from py_docx_creator.AbstractClasses import DocumentStyles, FontNames, ParagraphStyle, TextStyle
from py_docx_creator.CoreClasses import CoreDocumentStyle, CorePageStyle, \
    CoreTextStyle, CoreDocumentWriter, AlignParagraph, CoreParagraphStyle, CoreStyleManager


class NormalDocumentStyle(CoreDocumentStyle):
    """Стандартный стиль документа"""

    def __init__(self):
        super().__init__()
        self.document_style = DocumentStyles.Normal.value


@dataclass
class MainPageStyle(CorePageStyle):
    """Основной формат страницы"""
    top_margin: Pt | None = Pt(15)
    bottom_margin: Pt | None = Pt(10)
    left_margin: Pt | None = Pt(75)
    right_margin: Pt | None = Pt(75)


@dataclass
class MainParagraphStyle(CoreParagraphStyle):
    """Стиль основного текста"""
    alignment: AlignParagraph | None = AlignParagraph.JUSTIFY.value
    space_after: Pt | None = Pt(0)
    left_indent: Inches | None = Inches(-0.5)
    right_indent: Inches | None = Inches(-0.5)
    line_spacing: float | None = 1.15
    first_line_indent: Pt | None = 20


@dataclass
class HeaderParagraphStyle(CoreParagraphStyle):
    """Стиль для заголовков """
    alignment: AlignParagraph = AlignParagraph.CENTER.value
    left_indent: Inches = Inches(-0.5)
    right_indent: Inches = Inches(-0.5)


class MainDocumentWriter(CoreDocumentWriter):
    def __init__(self):
        super().__init__()


@dataclass
class MainTextStyle(CoreTextStyle):
    """Основной стиль текста"""
    size: Pt = Pt(10)
    name: str = FontNames.TimesNewRoman.value
    bold: bool = False


@dataclass
class HeaderTextStyle(CoreTextStyle):
    size: Pt = Pt(12)
    name: str = FontNames.TimesNewRoman.value
    bold = True


class FastWriter(CoreDocumentWriter):
    """Класс для быстрой записи в документ"""

    @classmethod
    def write(cls, document: Document, text: str, paragraph_style: Type[ParagraphStyle], text_style: Type[TextStyle],
              size: int = None, bold: bool = None, italic: bool = None, underline: bool = None,
              space_after: int = None, alignment: WD_PARAGRAPH_ALIGNMENT=None) -> None:
        """
        Метод записи в документ
        Аргументы:

            document: Document - документ для записи

            text: str - записываемый текст

            paragraph_style: CoreParagraphStyle - стиль параграфа

            text_style: CoreTextStyle - стиль текста

        Опциональные аргументы:

            Нижеперечисленные аргументы используются для быстрого изменения основных настроек стиля
            без необходимости прописывать отдельный dataclass

            size: int

            bold: bool

            italic: bool

            underline: bool

            space_after: int

            alignment: AlignParagraph.*

        """
        if size is not None:
            text_style.size = Pt(size)
        if bold is not None:
            text_style.bold = bold
        if italic is not None:
            text_style.italic = italic
        if underline is not None:
            text_style.underline = underline
        if space_after is not None:
            paragraph_style.space_before = Pt(space_after)
        if alignment is not None:
            paragraph_style.alignment = alignment.value


        paragraph = cls.add_paragraph_to_document(document)
        CoreStyleManager.PARAGRAPH_STYLE_MANAGER.apply_style(paragraph, paragraph_style)
        run = cls.add_run_to_paragraph(paragraph, text)
        CoreStyleManager.TEXT_STYLE_MANAGER.apply_style(run, text_style)



from abc import ABC, abstractmethod
from dataclasses import dataclass
from enum import Enum

from docx.document import Document
from docx.enum.text import WD_ALIGN_PARAGRAPH
from docx.shared import Pt, Inches
from docx.styles.styles import Styles
from docx.text.paragraph import Paragraph
from docx.text.run import Run


# region _________________Перечни для подстановки_________________

class FontNames(Enum):
    """Перечень наименований шрифтов"""
    TimesNewRoman = "Times New Roman"


class DocumentStyles(Enum):
    """Перечень стилей документа"""
    Normal = "Normal"


@dataclass
class AlignParagraph:
    LEFT: WD_ALIGN_PARAGRAPH = WD_ALIGN_PARAGRAPH.LEFT
    CENTER: WD_ALIGN_PARAGRAPH = WD_ALIGN_PARAGRAPH.CENTER
    RIGHT: WD_ALIGN_PARAGRAPH = WD_ALIGN_PARAGRAPH.RIGHT
    JUSTIFY: WD_ALIGN_PARAGRAPH = WD_ALIGN_PARAGRAPH.JUSTIFY
    DISTRIBUTE: WD_ALIGN_PARAGRAPH = WD_ALIGN_PARAGRAPH.DISTRIBUTE
    JUSTIFY_MED: WD_ALIGN_PARAGRAPH = WD_ALIGN_PARAGRAPH.JUSTIFY_MED
    JUSTIFY_HI: WD_ALIGN_PARAGRAPH = WD_ALIGN_PARAGRAPH.JUSTIFY_HI
    JUSTIFY_LOW: WD_ALIGN_PARAGRAPH = WD_ALIGN_PARAGRAPH.JUSTIFY_LOW
    THAI_JUSTIFY: WD_ALIGN_PARAGRAPH = WD_ALIGN_PARAGRAPH.THAI_JUSTIFY


# endregion _________________Перечни для подстановки_________________

# region _________________Управление документом_________________

class DocumentCreator(ABC):
    """Класс для создания, чтения, записи документа"""

    def __init__(self):
        self._file_name: str | None = None
        self._path_to_document: str | None = None
        self._document: Document | None = None

    @abstractmethod
    def create_document(self, file_name: str) -> None:
        """Создание документа"""
        pass

    @abstractmethod
    def load_document(self) -> None:
        """Загрузка уже имеющегося документа"""
        pass

    @abstractmethod
    def save_document(self) -> None:
        """Сохранение документа"""
        pass

    @property
    def file_name(self) -> str | None:
        return self._file_name

    @file_name.setter
    def file_name(self, value: str) -> None:
        self._file_name = value

    @property
    def path_to_document(self) -> str | None:
        return self._path_to_document

    @path_to_document.setter
    def path_to_document(self, value: str) -> None:
        self._path_to_document = value

    @property
    def document(self) -> Document:
        return self._document

    @document.setter
    def document(self, value: Document) -> None:
        self._document = value


class DocumentWriter(ABC):
    """Класс для наполнения документа"""

    @staticmethod
    @abstractmethod
    def add_paragraph_to_document(document: Document) -> Paragraph | None:
        """Добавление параграфа в документ"""
        pass

    @staticmethod
    @abstractmethod
    def add_run_to_paragraph(paragraph: Paragraph, text: str) -> Run | None:
        """Добавить текст в параграф"""
        pass

    @staticmethod
    @abstractmethod
    def add_page_break(document: Document) -> None:
        """Добавление разрыва страницы в документ"""
        pass


class DocumentStyle(ABC):
    """Стиль документа"""

    def __init__(self):
        self._document_style: str | None = None

    @property
    def document_style(self) -> str | None:
        """Стиль документа"""
        return self._document_style

    @document_style.setter
    def document_style(self, value: str) -> None:
        """Стиль документа"""
        self._document_style = value

    @abstractmethod
    def get_document_style(self, document: Document) -> Styles | None:
        """Получение стиля документа"""
        pass


# endregion _________________Управление документом_________________

# region _________________Стили_________________

@dataclass
class TextStyle(ABC):
    """
    Стиль текста.

    Атрибуты:
        size ( Pt | None): # размер шрифта

        name ( str | None): # наименование шрифта

        bold ( bool | None): # жирное начертание шрифта

        italic ( bool | None): # курсивное начертание шрифта

        underline ( bool | None): # подчеркнутое начертание шрифта

    """

    size: Pt | None  # размер шрифта
    name: str | None  # наименование шрифта
    bold: bool | None  # жирное начертание шрифта
    italic: bool | None  # курсивное начертание шрифта
    underline: bool | None  # подчеркнутое начертание шрифта


@dataclass
class PageStyle(ABC):
    """
    Стиль страницы.

        Атрибуты:
            top_margin ( Pt | None): # отступ сверху

            bottom_margin ( Pt | None): # отступ снизу

            left_margin ( Pt | None): # отступ слева

            right_margin ( Pt | None): # отступ справа

    """

    top_margin: Pt | None  # отступ сверху
    bottom_margin: Pt | None  # отступ снизу
    left_margin: Pt | None  # отступ слева
    right_margin: Pt | None  # отступ справа


@dataclass
class ParagraphStyle(ABC):
    """
    Стиль форматирования параграфа.

    Атрибуты:
        alignment (AlignParagraph | None): Выравнивание текста (влево, по центру, по ширине и т.п.).

        space_after (Pt | None): Отступ после параграфа.

        space_before (Pt | None): Отступ перед параграфом.

        left_indent (Inches | None): Отступ от левого края страницы.

        right_indent (Inches | None): Отступ от правого края страницы.

        line_spacing (float | None): Межстрочный интервал.

        first_line_indent (Pt | None): Отступ первой строки (красная строка).

        page_break_before (bool | None): Разрыв страницы перед параграфом.
    """

    alignment: AlignParagraph | None  # выравнивание
    space_after: Pt | None  # отступ до параграфа
    space_before: Pt | None  # отступ после параграфа
    left_indent: Inches | None  # отступ от левого края
    right_indent: Inches | None  # отступ от правого края
    line_spacing: float | None  # межстрочный интервал
    first_line_indent: Pt | None  # отступ красной строки
    page_break_before: bool | None  # разрыв страницы перед параграфом


# endregion _________________Стили_________________

# region _________________Управление стилями_________________
class PageStyleManager(ABC):
    @staticmethod
    @abstractmethod
    def apply_style(document: Document, style: PageStyle) -> None:
        """Применение стиля"""
        pass


class TextStyleManager(ABC):
    @staticmethod
    @abstractmethod
    def apply_style(run: Run, style: TextStyle) -> None:
        """Применение стиля"""
        pass


class ParagraphStyleManager(ABC):
    @staticmethod
    @abstractmethod
    def apply_style(paragraph: Paragraph, style: ParagraphStyle) -> None:
        """Применение стиля к параграфу"""
        pass


# endregion _________________Управление стилями_________________

# region _________________Централизированное управление менеджерами_________________

@dataclass
class StyleManager(ABC):
    PAGE_STYLE_MANAGER: PageStyleManager
    TEXT_STYLE_MANAGER: TextStyleManager
    PARAGRAPH_STYLE_MANAGER: ParagraphStyleManager
# endregion _________________Централизированное управление менеджерами_________________



from dataclasses import dataclass, fields
from typing import Any

from docx import Document
from docx.shared import Pt, Inches
from docx.styles.styles import Styles
from docx.text.paragraph import Paragraph
from docx.text.run import Run

from py_docx_creator.AbstractClasses import DocumentCreator, DocumentWriter, DocumentStyle, PageStyle, ParagraphStyle, \
    TextStyle, ParagraphStyleManager, TextStyleManager, PageStyleManager, AlignParagraph


# region _________________Управление документом_________________

class CoreDocumentCreator(DocumentCreator):

    def __init__(self):
        super().__init__()
        self._document = None
        self._file_name = None

    def create_document(self, file_name):
        self.document = Document()
        self.file_name = file_name

    def load_document(self):
        self.document = Document(self.path_to_document or self.file_name)

    def save_document(self):
        self.document.save(self.file_name)


class CoreDocumentWriter(DocumentWriter):

    @staticmethod
    def add_paragraph_to_document(document: Document) -> Paragraph:
        return document.add_paragraph()

    @staticmethod
    def add_run_to_paragraph(paragraph: Paragraph, text: str) -> Run:
        return paragraph.add_run(text)

    @staticmethod
    def add_page_break(document: Document) -> None:
        document.add_page_break()


class CoreDocumentStyle(DocumentStyle):

    def __init__(self):
        super().__init__()

    def get_document_style(self, document: Document) -> Styles:
        return document.style[f"{self.document_style}"]


# endregion _________________Управление документом_________________

# region _________________Стили_________________

@dataclass
class CoreTextStyle(TextStyle):
    """
        Стиль текста.

        Атрибуты:
            size ( Pt | None): # размер шрифта

            name ( str | None): # наименование шрифта

            bold ( bool | None): # жирное начертание шрифта

            italic ( bool | None): # курсивное начертание шрифта

            underline ( bool | None): # подчеркнутое начертание шрифта

        """

    size: Pt | None = None
    name: str | None = None
    bold: bool | None = None
    italic: bool | None = None
    underline: bool | None = None


@dataclass
class CorePageStyle(PageStyle):
    """
        Стиль страницы.

            Атрибуты:
                top_margin ( Pt | None): # отступ сверху

                bottom_margin ( Pt | None): # отступ снизу

                left_margin ( Pt | None): # отступ слева

                right_margin ( Pt | None): # отступ справа

        """

    top_margin: Pt | None = None
    bottom_margin: Pt | None = None
    left_margin: Pt | None = None
    right_margin: Pt | None = None


@dataclass
class CoreParagraphStyle(ParagraphStyle):
    """
    Стиль форматирования параграфа.

    Атрибуты:
        alignment (AlignParagraph | None): Выравнивание текста (влево, по центру, по ширине и т.п.).

        space_after (Pt | None): Отступ после параграфа.

        space_before (Pt | None): Отступ перед параграфом.

        left_indent (Inches | None): Отступ от левого края страницы.

        right_indent (Inches | None): Отступ от правого края страницы.

        line_spacing (float | None): Межстрочный интервал.

        first_line_indent (Pt | None): Отступ первой строки (красная строка).

        page_break_before (bool | None): Разрыв страницы перед параграфом.
    """
    alignment: AlignParagraph | None = None  # выравнивание
    space_after: Pt | None = None  # отступ до параграфа
    space_before: Pt | None = None  # отступ после параграфа
    left_indent: Inches | None = None  # отступ от левого края
    right_indent: Inches | None = None  # отступ от правого края
    line_spacing: float | None = None  # межстрочный интервал
    first_line_indent: Pt | None = None  # отступ красной строки
    page_break_before: bool | None = None  # разрыв страницы перед параграфом


# endregion _________________Стили_________________

# region _________________Управление стилями_________________

class CorePageStyleManager(PageStyleManager):
    @staticmethod
    def apply_style(document: Document, style: Any) -> None:
        """Применение стиля из dataclass к секциям документа."""

        for section in document.sections:
            for field in fields(style):
                setattr(section, field.name, getattr(style, field.name))


class CoreTextStyleManager(TextStyleManager):
    @staticmethod
    def apply_style(run: Run, style: Any) -> None:
        """Применение стиля"""
        for field in fields(style):
            setattr(run, field.name, getattr(style, field.name))


class CoreParagraphStyleManager(ParagraphStyleManager):
    @staticmethod
    def apply_style(paragraph: Paragraph, style: Any) -> None:
        """Применение стиля к параграфу"""

        paragraph_style = paragraph.paragraph_format
        for field in fields(style):
            setattr(paragraph_style, field.name, getattr(style, field.name))


class CoreStyleManager:
    PAGE_STYLE_MANAGER: CorePageStyleManager = CorePageStyleManager
    TEXT_STYLE_MANAGER: CoreTextStyleManager = CoreTextStyleManager
    PARAGRAPH_STYLE_MANAGER: CoreParagraphStyleManager = CoreParagraphStyleManager

# endregion _________________Управление стилями_________________
